%function beta_store = GDFM(cf,x,z,nx,reps,burn,L,maxtrys,yearlab)

%cf: Capital Flows Data
%x: Data for factor Extraction. It is a cell array. First level is for
%   variables (eg: x{1,1}: Output, x{1,2}: Interest Rates). Second level is
%   for different hierarchical levels (eg: x{1,1}{1,1}: Output in Region 1,
%   x{1,1}{1,2}: Output in Region 2)
%z: Data on known CF drivers
%nx: Cell Array, containing number of variables in each hierarchical level
%    for each variables (eg: nx{1,1}{1,1}: Number of cross-sections for Output in Region 1)

orth_flag = 0; %Normalize factors before VDs?
norm_flag = 1; %Normalize VDs to sum to 1?

T = size(cf,1);
N = size(cf,2);
M = size(x,2);
Z = size(z,2);

dim_beta = 1+M; %No of variables in the State Vector
for m=1:M
    dim_beta = dim_beta+size(nx{1,m},2);
end

dim_obs = N; %No of variables in the Observation Equation
for m=1:M
    for j=1:size(x{1,m},2)
        dim_obs = dim_obs+size(x{1,m}{1,j},2);
    end
end

%0: Priors, Initial Conditions, Starting Values

X = []; %Observation Eqn Data Vector
XX = []; %Observation Eqn Data Vector
for m=1:M
    XX{1,m} = [];
    for j=1:size(x{1,m},2)
    XX{1,m} = [XX{1,m} x{1,m}{1,j}(:,:)];
    end
    X = [X XX{1,m}];
end
X = [X cf];


F0 = zeros(dim_beta*L,dim_beta*L);
Lambda0 = zeros(dim_obs,dim_beta*L);
fid = 1;
for m=1:M
    s_loc = 1; %Data Locations for Global Factors
    f_loc = size(XX{1,m},2);
    for i=2:m
        s_loc = s_loc + size(XX{1,i-1},2);
        f_loc = f_loc + size(XX{1,i-1},2);
    end
    Loc_x{fid,1} = [s_loc:f_loc dim_obs-N+1:dim_obs];     
    fid = fid + 1;
    
    for r = 1:size(nx{1,m},2)
        ss_loc = s_loc; %Data Locations for 1st Level Regional Factors
        ff_loc = s_loc -1 + size(x{1,m}{1,r},2);
        for i=2:r
            ss_loc = ss_loc + size(x{1,m}{1,i-1},2);
            ff_loc = ff_loc + size(x{1,m}{1,i-1},2);
        end
        
        Loc_x{fid,1} = [ss_loc:ff_loc  dim_obs-N+1:dim_obs];
        fid = fid + 1;
    end
end
Loc_x{fid,1} = [dim_obs-N+1:dim_obs]; %Data Locations for CF Factor



%0-1: Factor Model
%0-1-1: Factor Model Transition Equation
beta0 = zeros(dim_beta*L,1); 
c_f = zeros(dim_beta*L,1);
F0 = [zeros(dim_beta,dim_beta*L);eye(dim_beta*(L-1)) zeros(dim_beta,dim_beta*(L-1))];

P00 = 100*eye(dim_beta*L);
Q = [100*eye(dim_beta) zeros(dim_beta,dim_beta*(L-1)); zeros(dim_beta,dim_beta*(L))];

%0-1-2: Factor Model Measurement Equation
Lambda = zeros(dim_obs,dim_beta*L);
Lambdaz = zeros(dim_obs,Z);
c = zeros(dim_obs,1);
R = 100*eye(dim_obs);
Q0=0.1;
Tq0=5;

%0-1-3: Factor Model Country-Specific Components
R0=0.1;
Tr0=5;

Phi = zeros(dim_obs,1);



%Principle Components for Priors and Initial Conditions

PCs = zeros(T,dim_beta);
X_o=X;

for n=1:N
    bols = z\cf(:,n);
    X_o(:,end-N+n) = cf(:,n)-z*bols;
    Lambdaz(end-N+n,:) = bols;
end

fac = extract(X_o(:,Loc_x{dim_beta,1}),1); %Extract Common CF Factor
for n=1:N
    bols = fac\cf(:,n); %IC for Loadings
    if n==1
        if bols<=0
            fac = -fac;
            bols = fac\cf(:,n);
        end
    end
    resid(:,n) = cf(:,n)-fac*bols;
    beta(n) = bols;
end
X_o(:,Loc_x{dim_beta,1}) = resid;
Lambda(Loc_x{dim_beta,1},dim_beta) = beta';
lags =ones(size(fac,1),1);
for l=1:L
    lags = [lags lag(fac,l)];
end
bols = lags\fac; %IC for AR and Variance Parameters
resid = fac-lags*bols;
Q(dim_beta,dim_beta) = (resid'*resid)/(size(fac,1)-size(lags,2));
c_f(dim_beta,:) = bols(1,:);
F0(dim_beta,[dim_beta:dim_beta:dim_beta*L])=bols(2:end,:)';
PCs(:,end) = fac;

for m=1:M
    loc = 1+(m-1)*(dim_beta-1)/2;
    data = X_o(:,Loc_x{loc,1});
    fac = extract(data,1); %Extract Global Factors
    resid = [];
    beta=[];
    for n=1:size(data,2)
        bols = fac\data(:,n); %IC for Loadings
        if n==1
            if bols<=0
                fac = -fac;
                bols = fac\data(:,n);
            end
        end
        resid(:,n) = data(:,n)-fac*bols;
        beta(n) = bols;
    end
    X_o(:,Loc_x{loc,1}) = resid;
    Lambda(Loc_x{loc,1},loc) = beta';
    lags =[];
    for l=1:L
        lags = [lags lag(fac,l)];
    end
    bols = lags\fac; %IC for AR and Variance Parameters
    resid = fac-lags*bols;
    Q(loc,loc) = (resid'*resid)/(size(fac,1)-size(lags,2));
    c_f(loc,:) = bols(1,:);
    F0(loc,[loc:dim_beta:loc+end-dim_beta])=bols(2:end,:)'; 
    PCs(:,loc) = fac;
end


for m=1:M
    for r = 1:size(nx{1,m},2)
        loc = 1+r+(m-1)*(dim_beta-1)/2;
        data = X_o(:,Loc_x{loc,1});
        fac = extract(data,1); %Extract Global Factors
        resid = [];
        beta=[];
        for n=1:size(data,2)
            bols = fac\data(:,n); %IC for Loadings
            if n==1 && r~=2 %Loads + on the 1st Varaible
                if bols<=0
                    fac = -fac;
                    bols = fac\data(:,n);
                end
            elseif n==2 && r==2 %Zero restriction on the 2nd Factor Loading
                % on the 1st variable for identification. Choice for the 2nd
                % factor is, 2nd one loads on the highes number of variables.
                if bols<=0
                    fac = -fac;
                    bols = fac\data(:,n);
                end
            end
            resid(:,n) = data(:,n)-fac*bols;
            beta(n) = bols;
        end
        X_o(:,Loc_x{loc,1}) = resid;
        if r == 2
            beta(1) = 0;
        end
        Lambda(Loc_x{loc,1},loc) = beta';
        lags =[];
        for l=1:L
            lags = [lags lag(fac,l)];
        end
        bols = lags\fac; %IC for AR and Variance Parameters
        resid(:,n) = fac-lags*bols;
        Q(loc,loc) = (resid(:,n)'*resid(:,n))/(size(fac,1)-size(lags,2));
        c_f(loc,:) = bols(1,:);
        F0(loc,[loc:dim_beta:loc+end-dim_beta])=bols(2:end,:)'; 
        PCs(:,loc) = fac;
    end
end

X_o = X;
for i=1:dim_obs
    bols = [PCs]\X(:,i); %MA and Variance Parameters
    resid = X(:,i)-[PCs]*bols;
    X_o(:,i) = resid;
    bols = lag(X(:,i),1)\X(:,i);
    resid = X(:,i)-lag(X(:,i),1)*bols;
    R(i,i) = (resid'*resid)/(size(X(:,i),1)-size(lag(X(:,i),1),2));
    Phi(i) = bols;
end
F = F0;
XX =[];
%for n=1:N
%vnames = strvcat('cf','y_g','y_ame','y_eu','y_asia','r_g','r_ame','r_eu','r_asia');
%r = ols(cf(:,n),[PCs(:,1:end-1)]);
%plt(r,vnames),prt(r,vnames),disp(n),pause,clc
%end

beta_store = zeros(T-1,dim_beta,reps-burn);
Lambda_store = zeros(dim_obs,dim_beta,reps-burn);
Lambdaz_store = zeros(dim_obs,Z,reps-burn);
Idios_store = zeros(T-1,dim_obs,reps-burn);
vardec_store = zeros(dim_obs,dim_beta+Z+1,reps-burn);
contr_store = zeros(T-1,dim_obs,dim_beta+Z+1,reps-burn);
XX = X(2:end,:);
Q = [1*eye(dim_beta) zeros(dim_beta,dim_beta*(L-1)); zeros(dim_beta,dim_beta*(L))];




%Gibbs sampling algorithm 
index=1:20:reps;
tic;
jgibbs=1;
igibbs=1;
while jgibbs<reps-burn
    if sum(igibbs==index)>0
        disp(sprintf('Iteration Number = %s of %s', num2str(igibbs), num2str(reps)));
    end
    
    %for i=1:dim_obs
    %XX(:,i) = X(2:end,i) - Phi(i)*X(1:end-1,i);
    %end
    
    %Step 1: Extract the Factors
    [beta]=carterkohn_gdfm(XX,z,c_f,Lambda,Lambdaz,F,R,Q,L,beta0,P00);
    
    %Step 2: Draw Factor constant and AR Coefficients
    for m=1:dim_beta
        yy = beta(:,m);
        xx = ones(size(beta(:,m),1),1);
        for l=1:L
            xx = [xx lag(beta(:,m),l)];
        end
        yy = yy(L+1:end,:);
        xx = xx(L+1:end,:);
        mstar=vec(xx\yy);  %ols
        xx=xx'*xx;
        ixx=xx\eye(cols(xx));
        vstar=kron(Q(m,m),ixx);
        %F(m,[m m+dim_beta])=[mstar+(randn(1,size(xx,2))*cholx(vstar))']';
        check=-1;
        tryx=1;
        while check<0 && tryx<maxtrys
            coeff=[mstar+(randn(1,size(xx,2))*cholx(vstar))']';
            CH=stability_gdfm(coeff(2:end),1,L);
            if CH==0
                check=10;
            else
                tryx=tryx+1;
            end
        end
        c_f(m) = coeff(:,1)';
        F(m,[m m+dim_beta]) = coeff(:,2:end);
    end
    
    %Step 3: Draw Factor Variances
    for m=1:dim_beta
        yy = beta(:,m);
        xx=[];
        for l=1:L
            xx = [xx lag(beta(:,m),l)];
        end
        yy = yy(L+1:end,:);
        xx = xx(L+1:end,:);
        error=yy-[F(m,[m m+dim_beta])*xx']';
        %Q(m,m)=IG(Tq0,Q0,error);
        Q(m,m)=1;
    end
    
    %Step 4: Draw Factor Loadings
    sign_r_loc = [];
    for i=1:dim_beta
        sign_r_loc = [sign_r_loc min(find(Lambda(:,i)~=0))];
    end
    for m=1:dim_obs
        loc = find(Lambda(m,:)~=0);
        yy = XX(:,m);
        xx = beta(:,loc);
        if m>dim_obs-N
            xx = [xx z(2:end,:)];
        end
        mstar=vec(xx\yy);  %ols
        xx=xx'*xx;
        ixx=xx\eye(cols(xx));
        vstar=kron(R(m,m),ixx);
        chk = -1;
        tryx = 1;
        while chk<0 && tryx<maxtrys
            if isempty(find(sign_r_loc==m))==1
                junk = [mstar+(randn(1,size(xx,2))*cholx(vstar))']';
                Lambda(m,loc) = junk(1:size(loc,2));
                if m>dim_obs-N
                    Lambdaz(m,:) = junk(size(loc,2)+1:end);
                end
                chk = 10;
            else
                junk=[mstar+(randn(1,size(xx,2))*cholx(vstar))']';
                flag = 1;
                locc = find(sign_r_loc==m);
                for r = 1:size(locc,2)
                    if junk(find(locc(r)==loc))<=0
                        flag = 0;
                    end
                end
                if flag == 1;
                    Lambda(m,loc) = junk(1:size(loc,2));
                    if m>dim_obs-N
                        Lambdaz(m,:) = junk(size(loc,2)+1:end);
                    end
                    chk = 10;
                else
                    tryx = tryx + 1;
                end
            end
        end
        if tryx==maxtrys
            disp(sprintf('Warning: Sign restriction Maxtry has been reached: Variable %s, Factor %s', num2str(m),num2str(r)));
        end
    end

    %Step 5: Draw Country/Variable Specific Component AR Coefficients
    Idios = zeros(T-1,dim_obs); %Idiosyncratic component
    for m=1:dim_obs
        loc = find(Lambda(m,:)~=0);
        yy = X(2:end,m);
        xx = beta(:,loc);
        error = yy-[(Lambda(m,loc)*xx'+Lambdaz(m,:)*z(2:end,:)')]';
        Idios(:,m) = error';
        yy = error(2:end,1);
        xx = error(1:end-1,1);
        mstar=vec(xx\yy);  %ols
        xxx=xx'*xx;
        ixx=xxx\eye(cols(xxx));
        vstar=kron(R(m,m),ixx);
        chk = -1;
        while chk<0
            Phi(m)=mstar+(randn(1,1)*cholx(vstar))';
            if Phi(m)<0.99;
                XX(:,m) = X(2:end,m) - Phi(m)*lag(error,1);
                chk = 10;
            end
        end
    %Step 6: Draw Country/Variable Specific Component Variances
        error = yy-xx*Phi(m);
        R(m,m) = IG(Tr0,R0,error);
    end
    
    if igibbs>burn
        jgibbs=jgibbs+1;
        beta_store(:,:,jgibbs) = beta(:,1:dim_beta);
        Lambda_store(:,:,jgibbs) = Lambda(:,1:dim_beta);
        Lambdaz_store(:,:,jgibbs) = Lambdaz;
        Idios_store(:,:,jgibbs) = Idios;
         
        var_decomp = zeros(dim_obs,dim_beta+Z+1);
        contr = zeros(T-1,dim_obs,dim_beta+Z+1);
        for m=1:dim_obs
            loc = find(Lambda(m,1:dim_beta)==0);
            yy = X(2:end,m);
            xx = beta(:,1:dim_beta);
            xx(:,loc) = 0;
            idios = Idios(:,m);
            coeff = Lambda(m,1:dim_beta);
            if m>size(X,2)-N
                coeff = [coeff Lambdaz(m,:)];
                xx = [xx z(2:end,:)];
            else
                coeff = [coeff zeros(size(z,2))];
                xx = [xx zeros(T-1,size(z,2))];
            end
            coeff = [coeff 1];
            xx = [xx idios];
            if orth_flag == 1 %Similar to Kose et al (2012), normalize factors before VDs
                xx_o = xx;
                loc = find(coeff(1,1:end-1)~=0);
                for i=1:size(loc,2)
                    yyy = xx_o(:,loc(1,i));
                    xxx = xx_o(:,loc(1,i+1:end));
                    xx_o(:,loc(1,i)) = yyy - xxx*(xxx\yyy);
                end
                coeff(:,loc) = (xx_o(:,loc)\yy)'; %Recalculate Loadings
                coeff(:,end) = 1; %Recalculate Idiosyncatic Part.
                xx_o(:,end) = yy-xx_o(:,loc(1,1:end-1))*(xx_o(:,loc(1,1:end-1))\yy);
                xx = xx_o; 
            end
            for i=1:size(coeff,2)
                var_decomp(m,i) = ((coeff(i)^2)*(var(xx(:,i))))/var(yy);
            end
            if norm_flag == 1
                for i=1:size(coeff,2)
                    junk = var_decomp(m,:);
                    var_decomp(m,i) = (junk(1,i))/sum(junk,2);
                end
            end
            for t = 1:T-1
                for i=1:size(coeff,2)
                contr(t,m,i) = coeff(i)*xx(t,i);
                end
            end
            %subplot(2,1,1), h = bar(yearlab(3:end),contr,'stacked'); hold on, plot(yearlab(3:end),yy,'-r','Linewidth',2); axis tight;
            %legend(strvcat(num2str([1:(size(coeff,2))]'),'Idios',sprintf('Variable No %s',num2str(m))),'Location','southwest'),set(legend,'FontSize',6);
            %subplot(2,1,2), plot(yearlab(3:end),yy-sum(contr,2),'-r','Linewidth',2); hold on, plot(yearlab(3:end),zeros(T-2,1),'-k'),axis tight;
            %pause,clc,clf
        end
        vardec_store(:,:,jgibbs) = var_decomp;
        contr_store(:,:,:,jgibbs) = contr;

    end
    igibbs=igibbs+1; 
end

qus = [.16, .5, .84];
beta_U_Interval = zeros(T-1,dim_beta);
beta_M_Interval = zeros(T-1,dim_beta);
beta_L_Interval = zeros(T-1,dim_beta);
Lambda_U_Interval = zeros(dim_obs,dim_beta);
Lambda_M_Interval = zeros(dim_obs,dim_beta);
Lambda_L_Interval = zeros(dim_obs,dim_beta);
Lambdaz_U_Interval = zeros(dim_obs,Z);
Lambdaz_M_Interval = zeros(dim_obs,Z);
Lambdaz_L_Interval = zeros(dim_obs,Z);
Idios_U_Interval = zeros(T-2,dim_obs);
Idios_M_Interval = zeros(T-2,dim_obs);
Idios_L_Interval = zeros(T-2,dim_obs);
vardec_U_Interval = zeros(dim_obs,dim_beta+Z+1);
vardec_M_Interval = zeros(dim_obs,dim_beta+Z+1);
vardec_L_Interval = zeros(dim_obs,dim_beta+Z+1);
contr_U_Interval = zeros(T-1,dim_obs,dim_beta+Z+1);
contr_M_Interval = zeros(T-1,dim_obs,dim_beta+Z+1);
contr_L_Interval = zeros(T-1,dim_obs,dim_beta+Z+1);

for i = 1:dim_beta
    for tt = 1:T-1
            junk = quantile(beta_store(tt,i,:),qus);
            beta_L_Interval(tt,i) = junk(1);
            beta_M_Interval(tt,i) = junk(2);
            beta_U_Interval(tt,i) = junk(3);
    end
    for tt = 1:dim_obs
            junk = quantile(Lambda_store(tt,i,:),qus);
            Lambda_L_Interval(tt,i) = junk(1);
            Lambda_M_Interval(tt,i) = junk(2);
            Lambda_U_Interval(tt,i) = junk(3);
    end
end
for i = 1:Z
    for tt = 1:dim_obs
            junk = quantile(Lambdaz_store(tt,i,:),qus);
            Lambdaz_L_Interval(tt,i) = junk(1);
            Lambdaz_M_Interval(tt,i) = junk(2);
            Lambdaz_U_Interval(tt,i) = junk(3);
    end
end
for i = 1:dim_obs
    for tt = 1:T-2
            junk = quantile(Idios_store(tt,i,:),qus);
            Idios_L_Interval(tt,i) = junk(1);
            Idios_M_Interval(tt,i) = junk(2);
            Idios_U_Interval(tt,i) = junk(3);
    end
end
for i = 1:dim_beta+Z+1
    for tt = 1:dim_obs
            junk = quantile(vardec_store(tt,i,:),qus);
            vardec_L_Interval(tt,i) = junk(1);
            vardec_M_Interval(tt,i) = junk(2);
            vardec_U_Interval(tt,i) = junk(3);
    end
end
for t = 1:T-1
    for i = 1:dim_beta+Z+1
        for tt = 1:dim_obs
                junk = quantile(contr_store(t,tt,i,:),qus);
                contr_L_Interval(t,tt,i) = junk(1);
                contr_M_Interval(t,tt,i) = junk(2);
                contr_U_Interval(t,tt,i) = junk(3);
        end
    end
end

set(0,'DefaultFigureWindowStyle','docked')

fnames = {'Y-gbl';'Y-ame';'Y-eu';'Y-asia';'R-gbl';'R-ame';'R-eu';'R-asia';'CF-Fac'};
vnames = {'Y-gbl';'Y-ame';'Y-eu';'Y-asia';'R-gbl';'R-ame';'R-eu';'R-asia';'VIX';'CF-Fac';'Idsyn'};

figure %Plot Factors
for i=1:dim_beta
    subplot(3,3,i), plot(yearlab(2:end),beta_M_Interval(:,i),yearlab(2:end),beta_U_Interval(:,i),'-r',yearlab(2:end),beta_L_Interval(:,i),'-r'),axis tight
    title(['Factor Plots: ' fnames{i}])
end

figure %Plot R2 from regressions of Factors on model variables
R2_store = zeros(dim_obs,dim_beta);
for i = 1:dim_obs
for n = 1:dim_beta
r = ols(X(2:end,i),beta_M_Interval(:,n));
R2_store(i,n) = r.rbar;
end
end
for i=1:size(find(Lambda(:,1:dim_beta)==0))
junk=find(Lambda(:,1:dim_beta)==0);
j=junk(i);
R2_store(j)=0;
end
for i=1:dim_beta
    subplot(3,3,i),bar(R2_store(:,i),1),axis tight, title(['Factor Loadings: ' fnames{i}])
end

figure %Plot Average R2 from a regression of Factors on Average PCFs
bar(mean(R2_store(end-N+1:end,:),1)),axis tight, set(gca, 'XTickLabel',fnames); ylim([0 1]), title('R2s from Univar. Regr. of each factor on mean PCFs')


figure %Plot Average Factor Contribution Graphs for Average PCFs
contr = zeros(T-1,size(contr_M_Interval,3));
for n=1:N
    contr = contr + squeeze(contr_M_Interval(:,dim_obs-N+n,:));
end
contr = contr/N;
cff = mean(X(2:end,dim_obs-N+1:end),2);
loc = find(contr(1,:)~=0);
subplot(2,1,1), bar(yearlab(2:end),contr(:,loc)); hold on, plot(yearlab(2:end),cff,'-r','Linewidth',2); axis tight;
legend(strvcat(num2str(loc(1,1:end-1)'),'Own Shock',sprintf('Variable No %s',num2str(m))),'Location','southwest'),set(legend,'FontSize',6);
subplot(2,1,2), plot(yearlab(2:end),cff-sum(contr(:,loc),2),'-r','Linewidth',2); hold on, plot(yearlab(2:end),zeros(T-1,1),'-k'),axis tight;


figure
for i=1:size(loc,2)
    subplot(4,3,i),bar(yearlab(2:end),contr(:,loc(i))); hold on, plot(yearlab(2:end),cff,'-r','Linewidth',2); axis tight;
    title([vnames{i} ' Contribution to PCFs - Average'])
end


figure %Plot Factor Contribution Graphs for PCFs
no_g = 3;
bar(mean(vardec_M_Interval(dim_obs-N+1:end,:),1)), title('Average Variance Decomp for PCFs'),set(gca, 'XTickLabel',vnames); ylim([0 1])
figure
for n = 1:round(N/no_g)
    subplot(round(N/(no_g*2)),2,n),bar(vardec_M_Interval(dim_obs-N+n,:)), title(['Variance Decomp for PCFs to Country: ' num2str(n)]),set(gca, 'XTickLabel',vnames); ylim([0 1])
end
if no_g>1
    figure
    for n = round(N/no_g)+1:2*round(N/no_g)
        nn = n-round(N/no_g);
        if n<=N
            subplot(round(N/(no_g*2)),2,nn),bar(vardec_M_Interval(dim_obs-N+n,:)), title(['Variance Decomp for PCFs to Country: ' num2str(n)]),set(gca, 'XTickLabel',vnames); ylim([0 1])
        end
    end
end
if no_g>2
    figure
    for n = 2*round(N/no_g)+1:3*round(N/no_g)
        nn = n-2*round(N/no_g);
        if n<=N
            subplot(round(N/(no_g*2)),2,nn),bar(vardec_M_Interval(dim_obs-N+n,:)), title(['Variance Decomp for PCFs to Country: ' num2str(n)]),set(gca, 'XTickLabel',vnames); ylim([0 1])
        end
    end
end


m = 1; %Plot Factor Contribution Graphs for a chosen variable m
figure
contr = squeeze(contr_M_Interval(:,m,:));
loc = find(contr(1,:)~=0);
for i=1:size(loc,2)
    subplot(2,2,i),bar(yearlab(2:end),contr(:,loc(i))); hold on, plot(yearlab(2:end),X(2:end,m),'-r','Linewidth',2); axis tight;
    title([vnames{loc(i)} ' Contribution to Variable No: ' num2str(m)])
end