%% Estimates a TVPVAR Model with USA Data using MCMC - Gibbs Sampling Method.

%load('data'); %Contains Data for USA
%DATAin = data(:,1:4); %Choose first 3 variables
DATAin = data(:,:); %Choose first 3 variables

%Inputs
HORIZONin=12;%forecast horizon
REPS=5000;%total reps
BURN=3500;%burn in
CHECKin=1;%1 to impose stability condition
Lin=3;   %number of lags in the VAR
MAXDRAWSin=5; %maximum number of tries to get stable draw if required
dateid = '';

%inputs
data=packr(DATAin);
reps=REPS;%total reps
burn=BURN;%burn in
CHECK=CHECKin;%1 to impose stability condition
HORIZON=HORIZONin;%forecast horizon
L=Lin;   %number of lags in the VAR
maxdraws=MAXDRAWSin; %maximum number of tries to get stable draw if required
if size(data,1)<50
    disp(strcat('Warning: Time Series is too short. The first 40 observations are used as a training sample leaving only:',num2str(size(data,1)-40)));
end
%
N=size(data,2);
%output=forecast density
forecast=zeros(reps-burn,HORIZON,N);

Y=data;
X=[];
for i=1:L
X=[X lag0(Y,i)];
end
X=[X ones(size(Y,1),1) ];
Y=Y(L+1:end,:);
X=X(L+1:end,:);


%step 1 set starting values and priors using a pre-sample of 10 years
T0=size(Y,1);
y0=Y(1:T0,:);
x0=X(1:T0,:);
b0=x0\y0;
e0=y0-x0*b0;
sigma0=(e0'*e0)/T0;

V0=kron(sigma0,inv(x0'*x0));


%priors for the variance of the transition equation
Q0=V0*T0*1e-03;  %prior for the variance of the transition equation error
P00=V0;                    % variance of the intial state vector  variance of state variable p[t-1/t-1]
beta0=vec(b0)';       % intial state vector   %state variable  b[t-1/t-1]

%priors and starting values for aij
C0=chol(sigma0);
C0=C0./repmat(diag(C0),1,N);
C0=inv(C0)';
PC0=10;


amatx=zeros(1,N*(N-1)/2);
j=1;
for i=1:N-1
    temp=C0(i+1,1:i);
amatx(1,j:j+cols(temp)-1)=temp;
D0{i}=eye(cols(temp))*(10^(-3));
j=j+cols(temp);
end
DD=D0;




%remove intial Sample
%Y=Y(T0+1:end,:);
%X=X(T0+1:end,:);
T=rows(X);
amat=repmat(amatx,T,1);
beta20=repmat(beta0,T,1);


%priors and starting values for the stochastic vol
hlast=(diff(Y).^2)+0.0001;
hlast=[hlast(1:2,:);hlast];  %rough intial guess for svol
g=ones(N,1);  %rough guess for the variance of the transition equation
g0=0.01^2;  %scale parameter for inverse gamma
Tg0=1;
mubar=log(diag(sigma0));
sigmabar=10;
%50 iterations to smooth initial estimates of stochastic volatility
epsilon=diff(Y);
epsilon=[epsilon(1,:);epsilon];
for m=1:50
  hnew=zeros(T+1,N);
  for i=1:N
      htemp=getsvol(hlast(:,i),g(i),mubar(i),sigmabar,epsilon(:,i));
      hnew(:,i)=htemp;
      
  end
hlast=hnew;
end


%initialise parameters
Q=Q0;


%Gibbs sampling algorithm 
index=1:100:reps;

tic;
mm=1;
m=1;
while mm<(reps-burn)+1
% if sum(m==index)>0
% disp(sprintf('Iteration Number= %s ', num2str(m)));
% end
%step 1 sample VAR coefficients
[beta2,error,roots,epsilon,problem]=carterkohnvar(Y,X,Q,amat,hlast,beta0,P00,L,CHECK,maxdraws);
if problem
    beta2=beta20;
else
    beta20=beta2;
end

% step 2 sample Q from the IW distribution
errorq=diff(beta2);
scaleQ=(errorq'*errorq)+Q0;
Q=iwpq(T+T0,invpd(scaleQ));

%step3 sample aij using the carter kohn algorithm
QA=zeros((N*(N-1))/2,(N*(N-1))/2);
j=1;
for i=2:N
    ytemp=error(:,i);
    xtemp=error(:,1:i-1)*-1;
    a0=C0(i,1:i-1);
    pa=PC0*diag(abs(a0));
    Qa=DD{i-1};
    [a1,trash]=carterkohn1(a0,pa,hlast(:,i),Qa,ytemp,xtemp);
    amat(:,j:j+cols(a1)-1)=a1;
    %sample Qa
    a1errors=diff(a1);
    scaleD1=(a1errors'*a1errors)+D0{i-1};
    DD{i-1}=iwpq(T+T0,invpd(scaleD1)); %draw from inverse Wishart
    QA(j:j+cols(a1)-1,j:j+cols(a1)-1)=DD{i-1};
    j=j+cols(a1);
end
  



 %sample stochastic vol for each epsilon using the MH algorithm
  hnew=zeros(T+1,N);
  for i=1:N
      htemp=getsvol(hlast(:,i),g(i),mubar(i),sigmabar,epsilon(:,i));
      hnew(:,i)=htemp;
       gerrors=diff(log(hnew(:,i)));
g(i)=IG(Tg0,g0,gerrors);  %draw from the inverse Gamma distribution
  end
hlast=hnew;

if sum(m==index)>0
disp(sprintf('Iteration Number= %s file=%s ', num2str(m),dateid));
end



if m>=burn
    if problem==0
    %Produce Forecast
    yhat=zeros(L+HORIZON,N);
    yhat(1:L,:)=Y(T-L+1:T,:);
    betahat=zeros(L+HORIZON,N*(N*L+1));
    betahat(L,:)=beta2(T,:);
    ahat=zeros(L+HORIZON,(N*(N-1))/2);
    ahat(L,:)=amat(T,:);
    hhat=zeros(L+HORIZON,N);
    hhat(L,:)=log(hlast(T+1,:));
    Qc=cholx(Q);
    gc=diag(sqrt(g));
    Qac=cholx(QA);
    for i=L+1:HORIZON+L
        hhat(i,:)=hhat(i-1)+randn(1,N)*gc;
        ahat(i,:)=ahat(i-1)+randn(1,(N*(N-1))/2)*Qac;
        sigmahat=chofac(N,ahat(i,:)')*diag(exp(hhat(i,:)))*chofac(N,ahat(i,:)')';
        A0=cholx(sigmahat);
        betahat(i,:)=betahat(i-1,:)+randn(1,N*(N*L+1))*Qc;
        %forecast
        xhat=[];
        for ii=1:L
            xhat=[xhat yhat(i-ii,:)];
        end
        xhat=[xhat 1];
        yhat(i,:)=xhat*reshape(betahat(i,:),N*L+1,N)+randn(1,N)*A0;
    end
            
     forecast(mm,:,:)=yhat(L+1:end,:);   
        
        
    
    
    mm=mm+1;
    end
    
end
m=m+1;
end
toc;

for n=1:N
    subplot(2,3,n)
    plot(1:size(Y,1)+HORIZON,[Y(:,n); nan(HORIZON,1)],1:size(Y,1)+HORIZON,[nan(size(Y(:,n))); quantile(squeeze(forecast(:,:,n)),.50)'],'-r',...
        1:size(Y,1)+HORIZON,[nan(size(Y(:,n))); quantile(squeeze(forecast(:,:,n)),.68)'],'--r',1:size(Y,1)+HORIZON,[nan(size(Y(:,n))); quantile(squeeze(forecast(:,:,n)),.32)'],'--r')
end